@@ -22,7 +22,9 @@ class SyntaxHighlight_GeSHi {
 	 */
 	public static function parserHook( $text, $args = array(), $parser ) {
 		self::initialise();
-		$text = trim( $text );
+		$text = rtrim( $text );
+		// Don't trim leading spaces away, just the linefeeds
+		$text = preg_replace( '/^\n+/', '', $text );
 		// Validate language
 		if( isset( $args['lang'] ) ) {
 			$lang = strtolower( $args['lang'] );
@@ -34,17 +36,18 @@ class SyntaxHighlight_GeSHi {
 		$geshi = self::prepare( $text, $lang );
 		if( !$geshi instanceof GeSHi )
 			return self::formatError( htmlspecialchars( wfMsgForContent( 'syntaxhighlight-err-language' ) ) );
-		// "Enclose" parameter
-		$enclose = isset( $args['enclose'] ) && $args['enclose'] == 'div'
-			? GESHI_HEADER_DIV
-			: GESHI_HEADER_PRE;
-		// Line numbers?
+
+		$enclose = self::getEncloseType( $args );
+
+		// Line numbers
 		if( isset( $args['line'] ) ) {
-			// Force <div> mode to maintain valid XHTML, see
-			// http://sourceforge.net/tracker/index.php?func=detail&aid=1201963&group_id=114997&atid=670231
-			$enclose = GESHI_HEADER_DIV;
 			$geshi->enable_line_numbers( GESHI_FANCY_LINE_NUMBERS );
 		}
+		// Highlighting specific lines
+		if( isset( $args['highlight'] ) ) {
+			$lines = self::parseHighlightLines( $args['highlight'] );
+			if ( count($lines) ) $geshi->highlight_lines_extra( $lines );
+		}
 		// Starting line number
 		if( isset( $args['start'] ) )
 			$geshi->start_line_numbers_at( $args['start'] );
@@ -60,13 +63,93 @@ class SyntaxHighlight_GeSHi {
 			return self::formatError( $err );
 		} else {
 			// Armour for Parser::doBlockLevels()
-			if( $enclose == GESHI_HEADER_DIV )
+			if( $enclose === GESHI_HEADER_DIV )
 				$out = str_replace( "\n", '', $out );
 			// Register CSS
 			$parser->mOutput->addHeadItem( self::buildHeadItem( $geshi ), "source-{$lang}" );
-			return '<div dir="ltr" style="text-align: left;">' . $out . '</div>';
+			if ( $enclose === GESHI_HEADER_NONE ) {
+				return '<span class="'.$lang.' source-'.$lang.'"> '.$out . '</span>';
+			} else {
+				return '<div dir="ltr" style="text-align: left;">' . $out . '</div>';
+			}
 		}
 	}
+	
+	/**
+	 * Take an input specifying a list of lines to highlight, returning
+	 * a raw list of matching line numbers.
+	 *
+	 * Input is comma-separated list of lines or line ranges.
+	 *
+	 * @input string
+	 * @return array of ints
+	 */
+	protected static function parseHighlightLines( $arg ) {
+		$lines = array();
+		$values = array_map( 'trim', explode( ',', $arg ) );
+		foreach ( $values as $value ) {
+			if ( ctype_digit($value) ) {
+				$lines[] = (int) $value;
+			} elseif ( strpos( $value, '-' ) !== false ) {
+				list( $start, $end ) = array_map( 'trim', explode( '-', $value ) );
+				if ( self::validHighlightRange( $start, $end ) ) {
+					for ($i = intval( $start ); $i <= $end; $i++ ) {
+						$lines[] = $i;
+					}
+				} else {
+					wfDebugLog( 'geshi', "Invalid range: $value\n" );
+				}
+			} else {
+				wfDebugLog( 'geshi', "Invalid line: $value\n" );
+			}
+		}
+		return $lines;
+	}
+	
+	/**
+	 * Validate a provided input range
+	 */
+	protected static function validHighlightRange( $start, $end ) {
+		// Since we're taking this tiny range and producing a an
+		// array of every integer between them, it would be trivial
+		// to DoS the system by asking for a huge range.
+		// Impose an arbitrary limit on the number of lines in a
+		// given range to reduce the impact.
+		$arbitrarilyLargeConstant = 10000;
+		return
+			ctype_digit($start) &&
+			ctype_digit($end) &&
+			$start > 0 &&
+			$start < $end &&
+			$end - $start < $arbitrarilyLargeConstant;
+	}
+
+	static function getEncloseType( $args ) {
+		// Since version 1.0.8 geshi can produce valid pre, but we need to check for it
+		if ( defined('GESHI_HEADER_PRE_VALID') ) {
+			$pre = GESHI_HEADER_PRE_VALID;
+		} else {
+			$pre = GESHI_HEADER_PRE;
+		}
+
+		// "Enclose" parameter
+		$enclose = $pre;
+		if ( isset( $args['enclose'] ) ) {
+			if ( $args['enclose'] === 'div' ) {
+				$enclose = GESHI_HEADER_DIV;
+			} elseif ( $args['enclose'] === 'none' ) {
+				$enclose = GESHI_HEADER_NONE;
+			}
+		}
+
+		if( isset( $args['line'] ) && $pre === GESHI_HEADER_PRE ) {
+			// Force <div> mode to maintain valid XHTML, see
+			// http://sourceforge.net/tracker/index.php?func=detail&aid=1201963&group_id=114997&atid=670231
+			$enclose = GESHI_HEADER_DIV;
+		}
+
+		return $enclose;
+	}
 
 	/**
 	 * Hook into Article::view() to provide syntax highlighting for
@@ -88,7 +171,7 @@ class SyntaxHighlight_GeSHi {
 			if( !$geshi->error() ) {
 				// Done
 				$output->addHeadItem( "source-$lang", self::buildHeadItem( $geshi ) );
-				$output->addHtml( "<div dir=\"ltr\">{$out}</div>" );
+				$output->addHTML( "<div dir=\"ltr\">{$out}</div>" );
 				return false;
 			}
 		}
@@ -128,7 +211,9 @@ class SyntaxHighlight_GeSHi {
 		$lang = $geshi->language;
 		$css[] = '<style type="text/css">/*<![CDATA[*/';
 		$css[] = ".source-$lang {line-height: normal;}";
-		$css[] = ".source-$lang li {line-height: normal;}";
+		$css[] = ".source-$lang li, .source-$lang pre {";
+		$css[] = "\tline-height: normal; border: 0px none white;";
+		$css[] = "}";
 		$css[] = $geshi->get_stylesheet( false );
 		$css[] = '/*]]>*/';
 		$css[] = '</style>';
@@ -154,7 +239,7 @@ class SyntaxHighlight_GeSHi {
 		if( $error )
 			$html .= "<p>{$error}</p>";
 		$html .= '<p>' . htmlspecialchars( wfMsgForContent( 'syntaxhighlight-specify' ) )
-			. ' <samp>&lt;source lang=&quot;html&quot;&gt;...&lt;/source&gt;</samp></p>'
+			. ' <samp>&lt;source lang=&quot;html4strict&quot;&gt;...&lt;/source&gt;</samp></p>'
 			. '<p>' . htmlspecialchars( wfMsgForContent( 'syntaxhighlight-supported' ) ) . '</p>'
 			. self::formatLanguages();
 		return "<div style=\"border: solid red 1px; padding: .5em;\">{$html}</div>";
